// @HEADER
// ***********************************************************************
// 
//               TSFCore: Trilinos Solver Framework Core
//                 Copyright (2004) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Michael A. Heroux (maherou@sandia.gov) 
// 
// ***********************************************************************
// @HEADER

// ////////////////////////////////////////////////////////////////////////////
// TSFCoreSerialVectorSpaceDecl.hpp

#ifndef TSFCORE_VECTOR_SPACE_SERIAL_DECL_HPP
#define TSFCORE_VECTOR_SPACE_SERIAL_DECL_HPP

#include "TSFCoreSerialVectorSpaceBase.hpp"

namespace TSFCore {

///
/** <tt>%VectorSpace</tt> Subclass for serial vectors and multi-vectors.
 *
 * The default copy constructor and assignment operators are allowed
 * since they have the correct semantics.
 */
template<class Scalar>
class SerialVectorSpace : public SerialVectorSpaceBase<Scalar> {
public:

	/** @name Constructors / initializers */
	//@{

	///
	/** Calls <tt>this->initialize()</tt>.
	 */
	SerialVectorSpace( int dim = 0 );

	///
	/** Initialize given the dimension of the vector space.
	 *
	 * @param  dim   [in] The dimension of the vector space.
	 */
	void initialize( int dim );

	//@}

	/** @name Overridden from VectorSpece */
	//@{

	/// Returns 0 if uninitialized
	Index dim() const;
	/// Returns a <tt>SerialVector</tt> object.
	Teuchos::RefCountPtr<Vector<Scalar> > createMember() const;
	///
	Teuchos::RefCountPtr< const VectorSpace<Scalar> > clone() const;

	//@}

private:

	int   dim_;

}; // end class SerialVectorSpace

} // end namespace TSFCore

#endif // TSFCORE_VECTOR_SPACE_SERIAL_DECL_HPP
