/* ***********************************************************************
// 
//           TSFExtended: Trilinos Solver Framework Extended
//                 Copyright (2004) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Michael A. Heroux (maherou@sandia.gov) 
// 
// **********************************************************************/

#include "TSFEpetraVector.hpp"
#include "TSFVector.hpp"

using namespace Teuchos;
using namespace TSFExtended;
using TSFCore::Index;


EpetraVector::EpetraVector(const RefCountPtr<Epetra_Vector>& vec,
                           const RefCountPtr<const TSFCore::EpetraVectorSpace>& vs)
   : TSFCore::EpetraVector(vec, vs) 
{;}

void EpetraVector::setElement(Index index, const double& value)
{
  epetra_vec()->ReplaceGlobalValues(1, const_cast<double*>(&value), 
                                    const_cast<int*>(&index));
}

void EpetraVector::addToElement(Index index, const double& value)
{
  epetra_vec()->SumIntoGlobalValues(1, const_cast<double*>(&value), 
                                    const_cast<int*>(&index));
}

const double& EpetraVector::getElement(Index index) const 
{
  const Epetra_BlockMap& myMap = epetra_vec()->Map();
  return (*epetra_vec())[myMap.LID(index)];
}

void EpetraVector::setElements(size_t numElems, const Index* globalIndices,
                               const double* values)
{
  Epetra_FEVector* vec = dynamic_cast<Epetra_FEVector*>(epetra_vec().get());
  int ierr = vec->ReplaceGlobalValues(numElems, globalIndices, values);
}

void EpetraVector::addToElements(size_t numElems, const Index* globalIndices,
                                 const double* values)
{
  Epetra_FEVector* vec = dynamic_cast<Epetra_FEVector*>(epetra_vec().get());
  int ierr = vec->SumIntoGlobalValues(numElems, globalIndices, values);
}

void EpetraVector::finalizeAssembly()
{
  Epetra_FEVector* vec = dynamic_cast<Epetra_FEVector*>(epetra_vec().get());
  vec->GlobalAssemble();
}

const Epetra_Vector& EpetraVector::getConcrete(const Vector<double>& tsfVec)
{
  const EpetraVector* epv 
    = dynamic_cast<const EpetraVector*>(tsfVec.ptr().get());
  TEST_FOR_EXCEPTION(epv==0, std::runtime_error,
                     "EpetraVector::getConcrete called on a vector that "
                     "could not be cast to an EpetraVector");
  return *(epv->epetra_vec());
}

Epetra_Vector& EpetraVector::getConcrete(Vector<double>& tsfVec)
{
  EpetraVector* epv 
    = dynamic_cast<EpetraVector*>(tsfVec.ptr().get());
  TEST_FOR_EXCEPTION(epv==0, std::runtime_error,
                     "EpetraVector::getConcrete called on a vector that "
                     "could not be cast to an EpetraVector");
  return *(epv->epetra_vec());
}

